package notify

import (
	// "compress/gzip"
	"compress/gzip"
	"fmt"
	"io"
	"strings"

	"github.com/gin-gonic/gin"
)

type Consumer struct {
	UserID string
	ClientID string
	conn io.WriteCloser
}

func NewConsumer(c *gin.Context)(*Consumer, error){
	user := c.Query("user")
	clientID := c.Query("client")
	if clientID == "" {
		return nil, fmt.Errorf("client id is empty")
	}
	conn, _, err := c.Writer.Hijack()
	if err != nil {
		return nil, err
	}
	consumer := Consumer{
		UserID: user,
		ClientID: clientID,
		conn: conn,
	}
	useGzip := false
	if strings.Contains(c.GetHeader("Accept-Encoding"), "gzip") {
		useGzip = true
	}
	
	err = consumer.init(useGzip)
	if err != nil {
		conn.Close()
	}
	return &consumer, err
}

func (c *Consumer)Write(msg string)error{
	_, err := c.conn.Write([]byte(fmt.Sprintf("data: %s\n\n", msg)))
	return err
}

func (c *Consumer)init(gz bool)error{
	if err := c.Write("HTTP/1.1 200 OK\r\nContent-Type: text/event-stream\r\n"); err != nil {
		return err
	}
	if err := c.Write("Vary: Accept-Encoding\r\n"); err != nil {
		return err
	}
	if gz {
		if err := c.Write("Content-Encoding: gzip\r\n"); err != nil {
			return err
		}
		c.conn = gzip.NewWriter(c.conn)
	}
	return c.Write("\r\n")
	return nil
}