package wechat

import (
	"crypto/sha1"
	"encoding/json"
	"fmt"
	"sort"
	"strings"
)

const (
	APIGetAccessToken = "https://api.weixin.qq.com/cgi-bin/token"
	APISendMessage    = "https://api.weixin.qq.com/cgi-bin/message/custom/send"
	APIGetUserInfo    = "https://api.weixin.qq.com/cgi-bin/user/info"
	APIGetGrantToken  = "https://api.weixin.qq.com/sns/oauth2/access_token"
	APICreateQRCode   = "https://api.weixin.qq.com/cgi-bin/qrcode/create"
)

type MessageType string

const (
	MessageTypeText = "text"
)

type TextMessage struct {
	Content string `json:"content"`
}

type Message struct {
	ToUser string      `json:"touser"`
	Type   MessageType `json:"msgtype"`
	Text   TextMessage `json:"text"`
}

func (m *Message) Show() {
	data, err := json.MarshalIndent(m, "", "\t")
	if err != nil {
		fmt.Println(err)
		return
	}
	fmt.Println(string(data))
}

//////
type MessageEntity struct {
	Signature string `form:"signature"`
	Timestamp string `form:"timestamp"`
	Nonce     string `form:"nonce"`
	EchoStr   string `form:"echostr"`
}

func (m *MessageEntity) CheckSignature(token string) bool {
	item := []string{token, m.Timestamp, m.Nonce}
	sort.Strings(item)
	itemByte := strings.Join(item, "")
	signature := fmt.Sprintf("%x", sha1.Sum([]byte(itemByte)))
	return signature == m.Signature
}

type FoucsRequest struct {
	ToUserName   string `xml:"ToUserName"`
	FromUserName string `xml:"FromUserName"`
	CreateTime   int64  `xml:"CreateTime"`
	MsgType      string `xml:"MsgType"`
	Event        string `xml:"Event"`
}

/*
{
    "subscribe": 1,
    "openid": "o6_bmjrPTlm6_2sgVt7hMZOPfL2M",
    "language": "zh_CN",
    "subscribe_time": 1382694957,
    "unionid": " o6_bmasdasdsad6_2sgVt7hMZOPfL",
    "remark": "",
    "groupid": 0,
    "tagid_list":[128,2],
    "subscribe_scene": "ADD_SCENE_QR_CODE",
    "qr_scene": 98765,
    "qr_scene_str": ""
}
*/
type UserInfo struct {
	Subscribe      int    `json:"subscribe"`
	OpenID         string `json:"openid"`
	Language       string `json:"language"`
	SubscribeTime  int64  `json:"subscribe_time"`
	UnionID        string `json:"unionid"`
	Remark         string `json:"remark"`
	GroupID        int    `json:"groupid"`
	LagIDList      []int  `json:"tagid_list"`
	SubscribeScene string `json:"subscribe_scene"`
	QRScene        int    `json:"qr_scene"`
	QRSceneStr     string `json:"qr_scene_str"`
	Nickname       string `json:"nickname"`
	Sex            int    `json:"sex"`
	Province       string `json:"province"`
	Country        string `json:"country"`
	HeadImage      string `json:"headimgurl"`
}

func (u *UserInfo) Show() {
	data, err := json.MarshalIndent(u, "", "\t")
	if err != nil {
		fmt.Println(err)
		return
	}
	fmt.Println(string(data))
}

type GrantToken struct {
	Token        string `json:"access_token"`
	ExpiresIn    int64  `json:"expires_in"`
	ExpiredAt    int64  `json:"expiredAt"`
	RefreshToken string `json:"refresh_token"`
	OpenID       string `json:"openid"`
	Scope        string `json:"scope"`
}

type Ticket struct {
	Ticket        string `json:"ticket"`
	ExpireSeconds int    `json:"expire_seconds"`
	URL           string `json:"url"`
}
